% Chapter 12, Fig.12.7
% Eric Dubois, updated 2019-01-15
%Plot square lattice Z^2 and identify orbits for 8-fold point group Gp
%about the point (-0.5,-0.5)

clear all, close all;

figure
m = -8:8; n=m;
[M,N]=meshgrid(m,n);
plot(M,N,'k.')  %plot points of Lambda
hold on 
mm = [-8 0 8]; nn=mm;
[MM,NN] = meshgrid(mm,nn);
plot(MM,NN,'ok')  % plot points of Gamma
axis equal
%plot the axes of symmetry
axis([-9 9 -9 9])
set(gca,'ydir','reverse');
plot([-9 9],[-.5 -.5], ':r')
plot([-.5 -.5],[-9 9],':r')
plot([-9 9],[-9 9],':r')
plot([-9 8],[8 -9],':r')
%Label the orbits A, B, ...
%orbit A
Arep = [-8 -1 0 7 8  -8 -1 0 7 8 -8 -1 0 7 8 -8 -1 0 7 8 -8 -1 0 7 8;...
    -8 -8 -8 -8 -8 -1 -1 -1 -1 -1 0 0 0 0 0 7 7 7 7 7 8 8 8 8 8];
for i = 1:size(Arep,2)
    text(Arep(1,i)+.15,Arep(2,i)+.05,'A','FontSize',7)
end
%orbit B
B0rep = [-7 -2 1 6 -7 -2 1 6 -7 -2 1 6 -7 -2 1 6 -7 -2 1 6 ;...
    -8 -8 -8 -8 -1 -1 -1 -1 0 0 0 0 7 7 7 7 8 8 8 8];
for i=1:size(B0rep,2)
    text(B0rep(1,i)+.15,B0rep(2,i)+.05,'B','FontSize',7)
end
B1rep = [-8 -1 0  7  8 -8 -1 0  7  8 -8 -1 0  7  8 -8 -1 0  7  8 ;...
         -7 -7 -7 -7 -7 -2 -2 -2 -2 -2 1 1 1 1 1 6 6 6 6 6];
for i=1:size(B1rep,2)
   text(B1rep(1,i)+.15,B1rep(2,i)+.05,'B','FontSize',7)
end    
%orbit C
Crep = [-7 -2 1 6 -7 -2 1 6 -7 -2 1 6 -7 -2 1 6 ;...
    -7 -7 -7 -7 -2 -2 -2 -2 1 1 1 1 6 6 6 6];
for i=1:size(Crep,2)
   text(Crep(1,i)+.15,Crep(2,i)+.05,'C','FontSize',7)
end 
%orbit D
D0rep = [-6 -3 2 5 -6 -3 2 5 -6 -3 2 5 -6 -3 2 5 -6 -3 2 5 ;...
    -8 -8 -8 -8 -1 -1 -1 -1 0 0 0 0 7 7 7 7 8 8 8 8];
for i=1:size(D0rep,2)
    text(D0rep(1,i)+.15,D0rep(2,i)+.05,'D','FontSize',7)
end
D1rep = [-8 -1 0 7 8 -8 -1 0 7 8 -8 -1 0 7 8 -8 -1 0 7 8 ;...
     -6 -6 -6 -6 -6 -3 -3 -3 -3 -3 2 2 2 2 2 5 5 5 5 5];
for i=1:size(D1rep,2)
   text(D1rep(1,i)+.15,D1rep(2,i)+.05,'D','FontSize',7)
end  
%orbit E
E0rep = [-7 -2 1 6 -7 -2 1 6 -7 -2 1 6 -7 -2 1 6; ...
    -6 -6 -6 -6 -3 -3 -3 -3 2 2 2 2 5 5 5 5];
for i=1:size(E0rep,2)
   text(E0rep(1,i)+.15,E0rep(2,i)+.05,'E','FontSize',7)
end 
E1rep = [-6 -3 2 5 -6 -3 2 5 -6 -3 2 5 -6 -3 2 5 ;...
    -7 -7 -7 -7 -2 -2 -2 -2 1 1 1 1 6 6 6 6];
for i=1:size(E1rep,2)
   text(E1rep(1,i)+.15,E1rep(2,i)+.05,'E','FontSize',7)
end 
%orbit F
Frep = [-6 -3 2 5 -6 -3 2 5  -6 -3 2 5  -6 -3 2 5   ;...
    -6 -6 -6 -6 -3 -3 -3 -3 2 2 2 2 5 5 5 5 ];
for i=1:size(Frep,2)
   text(Frep(1,i)+.15,Frep(2,i)+.05,'F','FontSize',7)
end 
%orbit G
G0rep = [ -8 -1 0 7 8 -8 -1 0 7 8 -8 -1 0 7 8 -8 -1 0 7 8 ;...
    -5 -5 -5 -5 -5 -4 -4 -4 -4 -4 3 3 3 3 3 4 4 4 4 4];
for i=1:size(G0rep,2)
    text(G0rep(1,i)+.15,G0rep(2,i)+.05,'G','FontSize',7)
end
G1rep = [ -5 -4 3 4 -5 -4 3 4 -5 -4 3 4 -5 -4 3 4 -5 -4 3 4 ; ...
    -8 -8 -8 -8 -1 -1 -1 -1 0 0 0 0 7 7 7 7 8 8 8 8  ];
for i=1:size(G1rep,2)
   text(G1rep(1,i)+.15,G1rep(2,i)+.05,'G','FontSize',7)
end 
% orbit H
H0rep = [-5 -4 3 4 -5 -4 3 4 -5 -4 3 4 -5 -4 3 4 ;...
    -7 -7 -7 -7 -2 -2 -2 -2 1 1 1 1 6 6 6 6];
for i=1:size(H0rep,2)
   text(H0rep(1,i)+.15,H0rep(2,i)+.05,'H','FontSize',7)
end 
H1rep = [-7 -2 1 6 -7 -2 1 6 -7 -2 1 6 -7 -2 1 6 ;...
    -5 -5 -5 -5 -4 -4 -4 -4 3 3 3 3 4 4 4 4 ];
for i=1:size(H1rep,2)
   text(H1rep(1,i)+.15,H1rep(2,i)+.05,'H','FontSize',7)
end  
% orbit I
I0rep = [-5 -4 3 4 -5 -4 3 4 -5 -4 3 4 -5 -4 3 4 ;...
    -6 -6 -6 -6 -3 -3 -3 -3 2 2 2 2 5 5 5 5 ];
for i=1:size(I0rep,2)
   text(I0rep(1,i)+.15,I0rep(2,i)+.05,'I','FontSize',7)
end
I1rep = [-6 -3 2 5 -6 -3 2 5 -6 -3 2 5 -6 -3 2 5 ;...
    -5 -5 -5 -5 -4 -4 -4 -4 3 3 3 3 4 4 4 4];
for i=1:size(I1rep,2)
   text(I1rep(1,i)+.15,I1rep(2,i)+.05,'I','FontSize',7)
end
% orbit J
J0rep = [-5 -4 3 4  -5 -4 3 4 -5 -4 3 4 -5 -4 3 4 ;...
    -5 -5 -5 -5 -4 -4 -4 -4 3 3 3 3 4 4 4 4 ];
for i=1:size(J0rep,2)
    text(J0rep(1,i)+.15,J0rep(2,i)+.05,'J','FontSize',7)
end
%Draw a polygon around one set of orbit representatives 
pent = polyshape([-.25 3.75 3.75 3 -.25],[-.25 -.25 3.5 3.5 .25]);
plot(pent,'FaceColor','white','FaceAlpha',0,'EdgeColor','blue');

set(gcf,'Color',[1 1 1]);
