%Chapter 7,  Fig 7.12
%Eric Dubois, updated 2018-12-13
%Plot a CIE xy chromaticity diagram with the triangle of physically
%realizable colors with an additive sRBG display, shown as colors on the
%diagram, with max(C_R,C_G,C_B)=1
% data files obtained from CVRL website
%Data files should be placed in a data directory on the MATLAB path.

close all; clear all;
%First create c_X, c_Y, c_Z image
%first chromaticities
x = 0:.001:.9;
y = .9 - x;
[X Y] = meshgrid(x,y);
im_xyz(:,:,1) = X;
im_xyz(:,:,2) = Y;
im_xyz(:,:,3) = ones(size(X))-X-Y;
%Convert to XYZ tristimulus values, assuming C_Y = 1
im_XYZ(:,:,1) = im_xyz(:,:,1)./im_xyz(:,:,2);
im_XYZ(:,:,2) = ones(size(X));
im_XYZ(:,:,3) = im_xyz(:,:,3)./im_xyz(:,:,2);
%Convert to sRGB tristimulus values
%1931 CIE XYZ to sRGB -- matrix from Poynton
A_XYZtosRGB = [3.240479 -1.537150 -0.498535; ...
              -0.969256  1.875992  0.041556; ...
               0.055648 -0.204043  1.057311];
im_RGB(:,:,1) = A_XYZtosRGB(1,1)*im_XYZ(:,:,1)+ ...
                A_XYZtosRGB(1,2)*im_XYZ(:,:,2)+ ...
                A_XYZtosRGB(1,3)*im_XYZ(:,:,3);
im_RGB(:,:,2) = A_XYZtosRGB(2,1)*im_XYZ(:,:,1)+ ...
                A_XYZtosRGB(2,2)*im_XYZ(:,:,2)+ ...
                A_XYZtosRGB(2,3)*im_XYZ(:,:,3);
im_RGB(:,:,3) = A_XYZtosRGB(3,1)*im_XYZ(:,:,1)+ ...
                A_XYZtosRGB(3,2)*im_XYZ(:,:,2)+ ...
                A_XYZtosRGB(3,3)*im_XYZ(:,:,3);   
%now limit to Maxwell triangle
im_RGB_max(:,:,1)= im_RGB(:,:,1)./(im_RGB(:,:,1)+im_RGB(:,:,2)+im_RGB(:,:,3));
im_RGB_max(:,:,2)= im_RGB(:,:,2)./(im_RGB(:,:,1)+im_RGB(:,:,2)+im_RGB(:,:,3));
im_RGB_max(:,:,3)= im_RGB(:,:,3)./(im_RGB(:,:,1)+im_RGB(:,:,2)+im_RGB(:,:,3));
%set all points with any coordinate out of [0,1] to white
coord_out = (im_RGB_max(:,:,1)<0) | (im_RGB_max(:,:,2)<0) | (im_RGB_max(:,:,3)<0) |...
    (im_RGB_max(:,:,1)>1) | (im_RGB_max(:,:,2)>1) | (im_RGB_max(:,:,3)>1);
coord_outn = ones(size(X))-coord_out;
im_RGB_max(:,:,1) = im_RGB_max(:,:,1).*coord_outn+coord_out;
im_RGB_max(:,:,2) = im_RGB_max(:,:,2).*coord_outn+coord_out;
im_RGB_max(:,:,3) = im_RGB_max(:,:,3).*coord_outn+coord_out;
%Now rescale so maximum tristimulus value is 1.
max_tristim = max(max(im_RGB_max(:,:,1),im_RGB_max(:,:,2)),im_RGB_max(:,:,3));
im_RGB_max(:,:,1)=im_RGB_max(:,:,1)./max_tristim;
im_RGB_max(:,:,2)=im_RGB_max(:,:,2)./max_tristim;
im_RGB_max(:,:,3)=im_RGB_max(:,:,3)./max_tristim;
%now do sRGB gamma correction
im_RGB_max_gc = 1.055*(im_RGB_max.^0.41666)-0.055;
im_RGB_max_gc(im_RGB_max<.0031308) = im_RGB_max(im_RGB_max<.0031308)*12.92;
%im_RGB_max_gc=sqrt(im_RGB_max);
figure;
image(x,y,im_RGB_max_gc) 
axis xy
hold on
%Add the chromaticity diagram
%read the CIE 1931 XYZ color matching functions.
%they are defined from 360 nm to 830 nm in steps of 1 nm.
xyz = dlmread('ciexyz31_1.txt',',');
lamxyz = xyz(:,1);
%compute chromaticities of the monochromatic lights
npts=size(xyz);
S = xyz(:,2:4) * [1 1 1]';
xyzc = [xyz(:,2)./S xyz(:,3)./S xyz(:,4)./S];
plot(xyzc(:,1),xyzc(:,2),'k');
xlabel('\it c_X'),ylabel('\it c_Y');
axis square;
axis([0 .9 0 .9]);
hold on
%line of purples
xl = linspace(xyzc(1,1), xyzc(npts(1),1));
yl = linspace(xyzc(1,2), xyzc(npts(1),2));
plot(xl,yl,'k');
set(gca,'fontname','times')          %set the plot font to Times

set(gcf,'Color',[1 1 1]);