%Chapter 8,  Fig 8.4
%Eric Dubois, updated 2018-12-14
%Plot the two-dimensional frequency response of the 
%discrete-domain Zhang and Wandell filters for S-CIELAB.
clear all, close all
% Set the samples per degree
ph = 1050; %number of pixels per picture height
vd = 2.5; %viewing distance in picture heights
thpx = (360/pi) * atan(1/(2*ph*vd)); % visual angle subtended by one pixel
PPD = 1/thpx;
% Parameters of the filters 
% Standard deviations in degrees from Zhang and Wandell
sO1 = [0.0283 0.1330 4.336];
sO2 = [0.0392 0.4940];
sO3 = [0.0536 0.3860];
%k parameters from Table 8.1
kO1 = [397.4 17.99  0.0169];
kO2 = [207.1 1.304];
kO3 = [110.8 2.136];
%w parameters from Table 8.1
wO1 = [1.0033 0.1144 -0.1176];
wO2 = [0.6167 0.3833];
wO3 = [0.5681 0.4319];
% Filter for component O1
E1 = 1.0;  %extent of filter for O1 in degrees
M1 = fix(E1*PPD/2)+1; % filter O1 goes from -M1 to M1
x1 = -M1:M1; y1 = x1; 
[X1, Y1] = meshgrid(x1,y1); %sample locations for filter O1
G11 = (kO1(1)/PPD^2)*exp(-(X1.^2+Y1.^2)/(sO1(1)*PPD)^2);
G12 = (kO1(2)/PPD^2)*exp(-(X1.^2+Y1.^2)/(sO1(2)*PPD)^2);
G13 = (kO1(3)/PPD^2)*exp(-(X1.^2+Y1.^2)/(sO1(3)*PPD)^2);
H11 = wO1(1)*G11 + wO1(2)*G12 + wO1(3)*G13;
g1 = 1/sum(sum(H11));
H11 = g1*H11;
[H11hat, Fx1,Fy1] = freqz2(H11,32,32,thpx);
figure;
mesh(Fx1,Fy1,abs(H11hat));	% generate the perspective plot
colormap([0 0 0]);	% use black only
xlabel('u (c/deg)'), ylabel('v (c/deg)');
set(gca,'ydir','reverse');
axis([-30 30 -30 30 0 1.2]);
set(gcf,'Color',[1 1 1]);
set(gca,'fontname','times')          %set the plot font to Times

% Filter for component O2
E2 = 1.;  %extent of filter for O2 in degrees
M2 = fix(E2*PPD/2)+1; % filter O2 goes from -M2 to M2
x2 = -M2:M2; y2 = x2; 
[X2, Y2] = meshgrid(x2,y2); %sample locations for filter O2
G21 = (kO2(1)/PPD^2)*exp(-(X2.^2+Y2.^2)/(sO2(1)*PPD)^2);
G22 = (kO2(2)/PPD^2)*exp(-(X2.^2+Y2.^2)/(sO2(2)*PPD)^2);
H22 = wO2(1)*G21 + wO2(2)*G22;
g2 = 1/sum(sum(H22));
H22 = g2*H22;
[H22hat, Fx2,Fy2] = freqz2(H22,32,32,thpx);
figure;
mesh(Fx2,Fy2,abs(H22hat));	% generate the perspective plot
colormap([0 0 0]);	% use black only
xlabel('u (c/deg)'), ylabel('v (c/deg)');
set(gca,'ydir','reverse');
axis([-30 30 -30 30 0 1.2]);
set(gcf,'Color',[1 1 1]);
set(gca,'fontname','times')          %set the plot font to Times

% Filter for component O3
E3 = 1.;  %extent of filter for O3 in degrees
M3 = fix(E3*PPD/2)+1; % filter O3 goes from -M3 to M3
x3 = -M3:M3; y3 = x3; 
[X3, Y3] = meshgrid(x3,y3); %sample locations for filter O3
G31 = (kO3(1)/PPD^2)*exp(-(X3.^2+Y3.^2)/(sO3(1)*PPD)^2);
G32 = (kO3(2)/PPD^2)*exp(-(X3.^2+Y3.^2)/(sO3(2)*PPD)^2);
H33 = wO3(1)*G31 + wO3(2)*G32;
g3 = 1/sum(sum(H33));
H33 = g3*H33;
[H33hat, Fx3,Fy3] = freqz2(H33,32,32,thpx);
figure;
mesh(Fx3,Fy3,abs(H33hat));	% generate the perspective plot
colormap([0 0 0]);	% use black only
xlabel('u (c/deg)'), ylabel('v (c/deg)');
set(gca,'ydir','reverse');
axis([-30 30 -30 30 0 1.2]);
set(gcf,'Color',[1 1 1]);
set(gca,'fontname','times')          %set the plot font to Times

%save('data/mdsp_ZWfilters.mat','H11','H22','H33');